import time
from threading import Event, Thread
from typing import Any, Callable, Optional

"""class RepeatedTimer(Timer):
    def run(self):
        while not self.finished.is_set():
            self.function(*self.args, **self.kwargs)
            self.finished.wait(self.interval)"""


class RepeatedTimer:
    def __init__(
        self,
        interval: int,
        function: Callable[[], None],
        *args: Optional[Any],
        **kwargs: Optional[Any]
    ):
        self.interval = interval
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.start = time.time()
        self.event = Event()
        self.thread = Thread(target=self._target)
        # self.thread.daemon = True
        self.thread.start()

    def _target(self):
        while not self.event.wait(self._time):
            self.function(*self.args, **self.kwargs)

    @property
    def _time(self):
        return self.interval - ((time.time() - self.start) % self.interval)

    def stop(self):
        self.event.set()
        self.thread.join()
